package org.apache.ojb.broker.platforms;

/* Copyright 2002-2004 The Apache Software Foundation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

import java.sql.PreparedStatement;
import java.sql.SQLException;
import java.sql.Types;

/**
 * SapDB specific Platform implementation
 * @author  Justin A. Stanczak
 * @author  Matthew Baird (mattb
 * @version $Id: PlatformSapdbImpl.java,v 1.10 2004/04/04 23:53:35 brianm Exp $
 */
public class PlatformSapdbImpl extends PlatformDefaultImpl
{
    public void setObjectForStatement(
            PreparedStatement ps,
            int index,
            Object value,
            int sqlType)
            throws SQLException
    {
        if (((sqlType == Types.VARBINARY) || (sqlType == Types.LONGVARBINARY))
                && (value instanceof byte[]))
        {
            byte buf[] = (byte[]) value;
            ps.setBytes(index, buf);
        }
        else
        {
            super.setObjectForStatement(ps, index, value, sqlType);
        }
    }

    /** Get join syntax type for this RDBMS - one on of the constants from JoinSyntaxType interface
     */
    public byte getJoinSyntaxType()
    {
        return ORACLE_JOIN_SYNTAX;
    }

    /**
     * Override default ResultSet size determination (rs.last();rs.getRow())
     * with select count(*) operation
     * SAP db doesn't let you use the .last, .getRow() mechanism (.getRow() will return -1)
     */
    public boolean useCountForResultsetSize()
    {
        return true;
    }

    public String createSequenceQuery(String sequenceName)
    {
        return "create sequence " + sequenceName;
    }

    public String nextSequenceQuery(String sequenceName)
    {
        return "select " + sequenceName + ".nextval from dual";
    }

    public String dropSequenceQuery(String sequenceName)
    {
        return "drop sequence " + sequenceName;
    }
    
    /* (non-Javadoc)
     * @see org.apache.ojb.broker.platforms.Platform#addPagingSql(java.lang.StringBuffer)
     */
    public void addPagingSql(StringBuffer anSqlString)
    {
        anSqlString.append(" ROWNO <= ? ");
    }

    /* (non-Javadoc)
     * @see org.apache.ojb.broker.platforms.Platform#bindPagingParameters(java.sql.PreparedStatement, int, int, int)
     */
    public int bindPagingParameters(PreparedStatement ps, int index, int startAt, int endAt) throws SQLException
    {
        
        ps.setInt(index, endAt - 1);    // IGNORE startAt !!
        index++;
        return index;
    }    
    
    /* (non-Javadoc)
     * @see org.apache.ojb.broker.platforms.Platform#supportsPaging()
     */
    public boolean supportsPaging()
    {
        return true;
    }    
}
