package org.apache.ojb.odmg;

import org.odmg.Implementation;

/* Copyright 2002-2004 The Apache Software Foundation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/**
 * Offers useful none odmg-standard methods of the odmg {@link org.odmg.Implementation} interface.
 * <p>
 * Note: All listed methods are <strong>not</strong> part of the standard ODMG-api -
 * they are special (proprietary) OJB extensions.
 * </p>
 *
 * @version $Id: ImplementationExt.java,v 1.1.2.2 2005/04/03 02:38:45 arminw Exp $
 */
public interface ImplementationExt extends Implementation
{
    /**
     * The used collection type class returned by OQL queries.
     *
     * @see org.apache.ojb.odmg.oql.EnhancedOQLQuery#execute()
     * @see org.odmg.OQLQuery#execute()
     * @return The collection class type
     */
    public Class getOqlCollectionClass();

    /**
     * Set the used collection type class returned by OQL queries.
     * <p/>
     * NOTE: Each specified class must implement interface {@link org.apache.ojb.broker.ManageableCollection}
     * to work proper with OJB.
     *
     * @param oqlCollectionClass The collection class used in OQL queries.
     */
    public void setOqlCollectionClass(Class oqlCollectionClass);

    /**
     * If the OJB implicit locking feature (see {@link TransactionExt#setImplicitLocking(boolean)}) is
     * enabled, this define the lock type of all implicit locked objects.
     * <p/>
     * If set to <em>true</em>, acquiring a write-lock on a given object x implies write locks on all
     * implicit locked objects.
     * <br/>
     * If set to <em>false</em>, in any case implicit read-locks are acquired.
     * Acquiring a read- or write lock on x thus allways results in implicit read-locks
     * on all associated objects.
     *
     * @param impliciteWriteLocks If <em>true</em> implicit write locks will enabled.
     */
    public void setImpliciteWriteLocks(boolean impliciteWriteLocks);

    /**
     * Is <em>true</em> when implicite write locks are enabled.
     *
     * @return <em>true</em> when implicit write locks are enabled.
     */
    boolean isImpliciteWriteLocks();

//    /**
//     * If set <em>true</em> the odmg implementation do it's best to find out the user intension, if set
//     * <em>false</em> OJB use an optimized mode and the user has to adhere strictly the odmg-api:
//     * <ul>
//     * <li>
//     * New objects can only be made persistent by using {@link org.odmg.Database#makePersistent(Object)}
//     * </li>
//     * <li>
//     * Only persistent objects can be locked with {@link org.odmg.Transaction#lock(Object, int)}.
//     * </li>
//     * <li>
//     * When deleting an object with {@link org.odmg.Database#deletePersistent(Object)} to reuse it
//     * within a transaction a call to {@link org.odmg.Database#makePersistent(Object)} is needed and
//     * field changes on objects marked as "deleted" are not allowed.
//     * </li>
//     * </ul>
//     * When running odmg in <em>safe-mode</em> these restrictions are "softened" and it's e.g. possible
//     * to persist new objects with {@link org.odmg.Transaction#lock(Object, int)}.
//     * <p/>
//     * The <em>optimized-mode</em> show a significant better performance, but needs strictness in using the API.
//     *
//     * @param safeMode Set <em>true</em> to enable the <em>safe-mode</em>, use <em>false</em> to enable
//     *                 the <em>optimized-mode</em>.
//     */
//    void setSafeMode(boolean safeMode);
//
//    /**
//     * Returns <em>true</em> if this class use the safe-mode for
//     * user interaction, else the optimized-mode is used.
//     */
//    boolean isSafeMode();
}
